import java.io.InputStream;
import awt.*;
import browser.*;
import net.www.html.*;

class ZoneData {
    Image imgs[];
    String imgName;
    boolean first;
    URL _url = null;
    int _x = 0, _y = 0, _width = 0, _height = 0;
    int nimgs = 0;
    int _curImgIdx = 0;
    String audioFile;
    boolean _playing;

    ZoneData(URL context, String dir, String iName, 
             int x, int y, int numImages, String url, 
             String zoneSound, MultiZoneAnimator parent, 
             boolean debug) {



	imgs = new Image[numImages];
        imgName = iName;
        audioFile = zoneSound;
        // _playing = false;

        if(debug==true) {
            System.err.println("ZoneData constructor:");
            System.err.println("\timage - " + iName + 
                " (x,y) = ("+x+","+ y+") " + "num images - " 
                + numImages);

            if(url!=null)
                System.err.println("URL: "+url);

            if(zoneSound!=null)
                 System.err.println("SOUND: " + zoneSound);
        }

	for (int i = 0; i < imgs.length; i++) {
	    Image im = parent.getImage(dir + "/" + imgName + 
                (i+1) + ".gif");
            if(debug==true)
                if(im!=null) 
                    System.err.println("ZoneData: got image - " + 
                        dir + "/" + imgName + (i+1) + ".gif");
                else
                    System.err.println("ZoneData: Did not get image - " + 
                        dir + "/" + imgName + (i+1) + ".gif");

	    if (im == null) { // raise exception here
		break;
	    }

            if(url!=null) {
                _url = new URL(url);
            }

	    imgs[nimgs++] = im;
            if(im != null && im.width > _width) {
		_width = im.width;
	    }
	    if(im != null && im.height > _height) {
		_height = im.height;
	    }
	}
        _x = x; _y = y; nimgs = numImages; 
        _curImgIdx = 0;
    }

    public int width() { return _width; }
    public int height() { return _height; }
    public int numImages() { return nimgs; }
    public void reset() { _curImgIdx = 0; }
    public URL url() { return _url; }
    public boolean playing() { return _playing; }
    public void setPlaying() { _playing = true; }
    public void resetPlaying() { _playing = false; }

    public Image getCurImage() { 
        if(nimgs==0)
            return null;

        Image img = imgs[_curImgIdx];
        if(++_curImgIdx==nimgs)
            _curImgIdx = 0;

        return img;
    }

    public boolean inside(int x, int y) {
        if(x >= _x && x <= (_x+_width) &&
           y >= _y && y <= (_y+_height))
            return true;
        return false;
    }


}

public
class MultiZoneAnimator extends Applet implements Runnable {
    int mode = 0;
    int appletW = 0, appletH = 0;

    int numZones;
    ZoneData zoneData[];
    ZoneData currentZone = null;

    // thread controls
    Thread animator = null;
    int pause;
    boolean threadSuspended = false;

    // image data
    Image	appMainImg, offscreenImg;
    Graphics	offscreen;

    // attributes from the doc
    String dataDir;
    String appMainImgName;

    boolean debug;

    public void init() {
        String s = getAttribute("DEBUG");
        debug = (s != null && s.equals("true")) ? true : false;
	appMainImgName = getAttribute("MAINIMG");
        dataDir = getAttribute("DATADIR");
        numZones = Integer.parseInt(getAttribute("NUMZONES"));
        zoneData = new ZoneData[numZones];
	appMainImg = getImage(dataDir + "/" + appMainImgName);

        if(debug==true) {
            if(appMainImg!=null)
                System.err.println("Got Main Image - " 
                    + dataDir + "/" + appMainImgName);
            else System.err.println("Did not get main Image - " 
                    + dataDir + "/" + appMainImgName);
        }

        if(appMainImg!=null) {
            appletW = appMainImg.width;
            appletH = appMainImg.height;
	    resize(appletW, appletH);
        }

	// get the data for all zones
        for(int idx=0; idx<numZones; idx++) {
            String imgName = getAttribute("ZONE"+(idx+1)+"IMG"); 
            String zoneURL = getAttribute("ZONE"+(idx+1)+"URL");
            String zoneSound = getAttribute("ZONE"+(idx+1)+"SOUND");
            int numImages = Integer.parseInt(
                getAttribute("ZONE"+(idx+1)+"NUMIMGS"));
            int x = Integer.parseInt(getAttribute("ZONE"+(idx+1)+"X"));
            int y = Integer.parseInt(getAttribute("ZONE"+(idx+1)+"Y"));
            zoneData[idx] = new ZoneData(documentURL, dataDir, imgName, 
                x, y, numImages, zoneURL, zoneSound, this, debug);
            appletW = (appletW > zoneData[idx].width()) ? 
                appletW : zoneData[idx].width();
            appletH = (appletH > zoneData[idx].height()) ? 
                appletH : zoneData[idx].height();
         }

    }

    public void run() {
    boolean first = true;
	// get the main image
    //    if(appletW==0 || appletH==0)
    //        return;

        currentZone = zoneData[0];

        // dump debug info and quit
        if(debug==true)
            dumpDebugInfo();

        int currentIdx = 0;

        while(animator != null) {
	if ( first == true ) {
            currentZone = zoneData[currentIdx];
            if(debug==true)
                System.err.println("Current Zone Idx = "+currentIdx);

            animateZone();
	    Thread.sleep(1000); // sleep for one seconds
            if ( currentIdx == numZones ) first = false;
	    ++currentIdx;
	  } 
            currentZone = zoneData[4];
            animateZone();
            Thread.sleep(1000); // sleep for one seconds
	}
    }

    private void dumpDebugInfo() {
        System.err.println("Main Image = "+appMainImgName);
        System.err.println("# Zones = "+numZones);
	System.err.println("Data Dir = "+dataDir);
        for(int idx=0; idx < numZones; idx++) {
            System.err.println("Zone "+idx+" : ");
            System.err.print("\t# Images = "+
                zoneData[idx].numImages()+" ,");
            System.err.print(" Image - "+zoneData[idx].imgName+" ,");
            System.err.println("at ("+zoneData[idx]._x+
                ","+zoneData[idx]._y+")");
        }
    }

    public void animateZone() {

        if(currentZone.playing()==true)
            return;
        else currentZone.setPlaying();

        currentZone.reset();

        // start playing audio
        InputStream is = null;
        
        if(currentZone.audioFile!=null) {
/* use this if playing long sounds
            is = getAudioStream(new URL(dataDir+ "/" + currentZone.audioFile));
            if(is!=null) 
                startPlaying(is);
*/
            play(dataDir+ "/" + currentZone.audioFile);
        }

        for(int curIdx=0; curIdx < currentZone.numImages(); curIdx++) {
            repaint();
            Thread.sleep(200);
        }

/* use this if playing long sounds
        try {
	    while(is!=null && is.available() > 0) {
		Thread.sleep(200);
	    }
        } catch(Exception e) {
        }

        if(is!=null) {
            stopPlaying(is); 
            is.close();
        }
*/
        currentZone.resetPlaying();
    }

    public void paint(Graphics g) {
        if(offscreenImg!=null)
	    g.drawImage(offscreenImg, 0, 0);
    }

    public void update(Graphics g) {
        if(appletW==0 || appletH==0)
             return;

	if(offscreenImg == null) {
	    offscreenImg = createImage(appletW, appletH);
	    offscreen = new Graphics(offscreenImg);
	    offscreen.setForeground(Color.lightGray);
	    offscreen.fillRect(0, 0, width, height);
            if(appMainImg!=null)
                offscreen.drawImage(appMainImg, 0, 0);
	    g.drawImage(offscreenImg, 0, 0);
	}

        if(appMainImg!=null) {
            Image curimg = currentZone.getCurImage();
            int x = currentZone._x, y = currentZone._y;

            if(debug==true)
                System.err.println("Drawing @ ("+ x+","+ y+")");

            // clean the area in offscreen to
            // the main image contents
            offscreen.fillRect(x, y, curimg.width, curimg.height);
            offscreen.clipRect(x, y, curimg.width, curimg.height);
            offscreen.drawImage(appMainImg, 0, 0);

            // paint the new image into the 
            // area in offscrenn
            offscreen.drawImage(curimg, x, y);
            offscreen.clearClip();

            // transfer the data from offscreen to
            // the screen
            g.clipRect(x, y, curimg.width, curimg.height);
            g.drawImage(offscreenImg, 0, 0);
            g.clearClip();
	}
    }

    public void start() {
	if (animator == null) {
	    animator = new Thread(this);
	    animator.start();
	}
    }

    public void stop() {
	animator = null;
    }

    public void mouseMove(int x, int y) {
        // check out which zone the URL corresponds to
        for(int i=0; i < numZones; i++) {
            if(zoneData[i].inside(x,y)) {
                currentZone = zoneData[i];
              	animateZone();
               	break;
            }
        }
    }
    public void mouseDown(int x, int y) {
        // check out which zone the URL corresponds to
        for(int i=0; i < numZones; i++) {
            if(zoneData[i].inside(x,y)) {
                if(zoneData[i].url()!=null) {
                    showDocument(zoneData[i].url());
                }
                break;
            }
        }
/*        
        if (threadSuspended)
            animator.resume();
	else
            animator.suspend();
        threadSuspended = !threadSuspended;
*/
    }

}


